"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cherow_1 = require("cherow");
const fs_1 = require("fs");
const util_1 = require("util");
const exportable_rules_1 = require("./exportable-rules");
function default_1(path) {
    const cachedExpressions = [];
    return util_1.promisify(fs_1.readFile)(path, { encoding: "utf8" })
        .then(context => {
        return cherow_1.parse(context, { module: true });
    })
        .then((program) => {
        let hasEnabledExportExtension = false;
        const result = program.body.reduce((acc, node) => {
            let ruleSet;
            try {
                ruleSet = exportable_rules_1.ExportableRuleSet.find(r => r.condition(node));
            }
            catch (_a) {
                ruleSet = null;
            }
            if (!ruleSet) {
                cachedExpressions.push(node);
                return acc;
            }
            else {
                if (ruleSet.rules.enablesExportExtension) {
                    hasEnabledExportExtension = true;
                }
                if (!hasEnabledExportExtension &&
                    ruleSet.rules.isExportExtension) {
                    return acc;
                }
                if (ruleSet.rules.override) {
                    return ruleSet.extract(node, cachedExpressions);
                }
            }
            return [...acc, ...ruleSet.extract(node, cachedExpressions)];
        }, []);
        return result;
    })
        .catch(err => {
        // tslint:disable-next-line: no-console
        console.error(err);
        return Promise.resolve([]);
    });
}
exports.default = default_1;
